@@ -2260,6 +2260,17 @@ void MySQL_Connection::async_free_result() {
 	}
 }
 
+// This function check if autocommit=0 and if there are any savepoint.
+// this is an attempt to mitigate MySQL bug https://bugs.mysql.com/bug.php?id=107875
+bool MySQL_Connection::AutocommitFalse_AndSavepoint() {
+	bool ret=false;
+	if (IsAutoCommit() == false) {
+		if (get_status(STATUS_MYSQL_CONNECTION_HAS_SAVEPOINT) == true) {
+			ret = true;
+		}
+	}
+	return ret;
+}
 
 bool MySQL_Connection::IsActiveTransaction() {
 	bool ret=false;
@@ -2275,12 +2286,13 @@ bool MySQL_Connection::IsActiveTransaction() {
 				ret = true;
 			}
 		}
-		if (ret == false) {
-			if (get_status(STATUS_MYSQL_CONNECTION_HAS_SAVEPOINT)) {
-				// there are savepoints
-				ret = true;
-			}
-		}
+		// in the past we were incorrectly checking STATUS_MYSQL_CONNECTION_HAS_SAVEPOINT
+		// and returning true in case there were any savepoint.
+		// Although flag STATUS_MYSQL_CONNECTION_HAS_SAVEPOINT was not reset in
+		// case of no transaction, thus the check was incorrect.
+		// We can ignore STATUS_MYSQL_CONNECTION_HAS_SAVEPOINT for multiplexing
+		// purpose in IsActiveTransaction() because it is also checked
+		// in MultiplexDisabled()
 	}
 	return ret;
 }
@@ -2520,7 +2532,7 @@ void MySQL_Connection::ProcessQueryAndSetStatusFlags(char *query_digest_text) {
 			}
 		}
 	} else {
-		if (
+		if ( // get_status(STATUS_MYSQL_CONNECTION_HAS_SAVEPOINT) == true
 			(
 				// make sure we don't have a transaction running
 				// checking just for COMMIT and ROLLBACK is not enough, because `SET autocommit=1` can commit too
@@ -2529,9 +2541,9 @@ void MySQL_Connection::ProcessQueryAndSetStatusFlags(char *query_digest_text) {
 				( (mysql->server_status & SERVER_STATUS_IN_TRANS) == 0 )
 			)
 			||
-			!strcasecmp(query_digest_text,"COMMIT")
+			(strcasecmp(query_digest_text,"COMMIT") == 0)
 			||
-			!strcasecmp(query_digest_text,"ROLLBACK")
+			(strcasecmp(query_digest_text,"ROLLBACK") == 0)
 		) {
 			set_status(false, STATUS_MYSQL_CONNECTION_HAS_SAVEPOINT);
 		}
@@ -2616,7 +2628,13 @@ int MySQL_Connection::async_send_simple_command(short event, char *stmt, unsigne
 	if (MyRS) {
 		// this is a severe mistake, we shouldn't have reach here
 		// for now we do not assert but report the error
-		proxy_error("Retrieved a resultset while running a simple command. This is an error!! Simple command: %s\n", stmt);
+		// PMC-10003: Retrieved a resultset while running a simple command using async_send_simple_command() .
+		// async_send_simple_command() is used by ProxySQL to configure the connection, thus it
+		// shouldn't retrieve any resultset.
+		// A common issue for triggering this error is to have configure mysql-init_connect to
+		// run a statement that returns a resultset.
+		proxy_error2(10003, "PMC-10003: Retrieved a resultset while running a simple command. This is an error!! Simple command: %s\n", stmt);
+		return -2;
 	}
 	if (async_state_machine==ASYNC_QUERY_END) {
 		compute_unknown_transaction_status();
