@@ -6,9 +6,12 @@
 #include <cstring>
 #include <memory>
 #include <openssl/ssl.h>
+
 #include "proxysql_structs.h"
-#include "proxysql_glovars.hpp"
 #include "sqlite3db.h"
+#include "MySQL_LDAP_Authentication.hpp"
+
+MySQL_LDAP_Authentication* GloMyLdapAuth = nullptr;
 
 int main() {
 	SQLite3DB::LoadPlugin(NULL);
