@@ -1848,7 +1848,7 @@ func (node *DataNodeImpl) CloseConnection() bool {
 	if node.Connection != nil {
 		err := node.Connection.Close()
 		if err != nil {
-			panic(err.Error())
+			log.Error(err.Error())
 			return false
 		}
 		return true
